@@ -6,7 +6,7 @@ import worlds._bizhawk as bizhawk
 from BaseClasses import ItemClassification
 from NetUtils import ClientStatus
 from worlds._bizhawk.client import BizHawkClient
-from .data import data, POKEDEX_OFFSET, POKEDEX_COUNT_OFFSET, FLY_UNLOCK_OFFSET
+from .data import data, POKEDEX_OFFSET, POKEDEX_COUNT_OFFSET, FLY_UNLOCK_OFFSET, GRASS_OFFSET
 from .items import item_const_name_to_id
 from .options import Goal, ProvideShopHints
 
@@ -14,6 +14,8 @@ if TYPE_CHECKING:
     from worlds._bizhawk.context import BizHawkClientContext
 
 EVENT_BYTES = math.ceil(max(data.event_flags.values()) / 8)
+DEX_BYTES = math.ceil(len(data.pokemon) / 8)
+GRASS_BYTES = math.ceil(sum(len(tiles) for tiles in data.grass_tiles.values()) / 8)
 
 TRACKER_EVENT_FLAGS = [
     "EVENT_GOT_KENYA",
@@ -182,6 +184,7 @@ class PokemonCrystalClient(BizHawkClient):
     phone_trap_locations: list[int]
     current_map: list[int]
     last_death_link: float
+    grass_location_mapping: dict[str, int]
 
     def initialize_client(self) -> None:
         self.local_checked_locations = set()
@@ -196,6 +199,7 @@ class PokemonCrystalClient(BizHawkClient):
         self.phone_trap_locations = list()
         self.current_map = [0, 0]
         self.last_death_link = 0
+        self.grass_location_mapping = dict()
 
     async def validate_rom(self, ctx: "BizHawkClientContext") -> bool:
         from CommonClient import logger
@@ -273,6 +277,8 @@ class PokemonCrystalClient(BizHawkClient):
         else:
             self.goal_flag = data.event_flags["EVENT_BEAT_RED"]
 
+        self.grass_location_mapping = ctx.slot_data["grass_location_mapping"]
+
         try:
 
             # Scout the locations that can be hinted if provide hints is turned on
@@ -315,8 +321,9 @@ class PokemonCrystalClient(BizHawkClient):
             read_result = await bizhawk.guarded_read(
                 ctx.bizhawk_ctx,
                 [(data.ram_addresses["wEventFlags"], EVENT_BYTES, "WRAM"),  # Flags
-                 (data.ram_addresses["wArchipelagoPokedexCaught"], 0x20, "WRAM"),
-                 (data.ram_addresses["wArchipelagoPokedexSeen"], 0x20, "WRAM")],
+                 (data.ram_addresses["wArchipelagoPokedexCaught"], DEX_BYTES, "WRAM"),
+                 (data.ram_addresses["wArchipelagoPokedexSeen"], DEX_BYTES, "WRAM"),
+                 (data.ram_addresses["wArchipelagoGrassFlags"], GRASS_BYTES, "WRAM")],
                 [overworld_guard]
             )
 
@@ -325,6 +332,7 @@ class PokemonCrystalClient(BizHawkClient):
 
             pokedex_caught_bytes = read_result[1]
             pokedex_seen_bytes = read_result[2]
+            grass_cut_bytes = read_result[3]
 
             game_clear = False
             local_checked_locations = set()
@@ -382,6 +390,15 @@ class PokemonCrystalClient(BizHawkClient):
                         dex_number = (byte_i * 8 + i) + 1
                         local_pokemon["seen"].append(dex_number)
 
+            for byte_i, byte in enumerate(grass_cut_bytes):
+                for i in range(8):
+                    if byte & (1 << i):
+                        location_id = (byte_i * 8 + i) + GRASS_OFFSET
+                        if str(location_id) in self.grass_location_mapping:
+                            location_id = self.grass_location_mapping[str(location_id)]
+                        if location_id in ctx.server_locations:
+                            local_checked_locations.add(location_id)
+
             if local_pokemon != self.local_pokemon and ctx.slot is not None:
                 await ctx.send_msgs([{
                     "cmd": "Set",
